function uptime
% UPTIME returns the current uptime (time elapsed) since the current MATLAB
% process has started. The return format is:
% MATLAB up DD days, HH:MM:SS

% Get current MATLAB process identifier
pid = num2str(feature('getpid'));
if isunix
    [status, out, command] = nixGetTime(pid);
else
    [status, out, command] = winGetTime(pid);
end
if (status == 0)
    if isunix
        [days,hhmmss] = nixParseTime(out);
    else
        [days,hhmmss] = winParseTime(out);
    end
    if days == 1
        % Say day or days?
        daymsg = 'day';
    else
        daymsg = 'days';
    end
    upmsg = sprintf('MATLAB up %d %s, %s',days,daymsg,hhmmss);
    disp(upmsg);
else
    error('Got error status "%d" when issuing command "%s"',status,command);
end
end

function [status, out, command] = nixGetTime(pid)
% On UNIX, we use ps utility
command = sprintf('%s %s %s','ps -p',pid,'-o etime');
[status, out] = system(command);
end

function [status, out, command] = winGetTime(pid)
% On Windows, we use VBS
filename = [tempname,'.vbs'];
fid = fopen(filename, 'w');
wuptime = {
    'Set colProcessList = GetObject("winmgmts:\\.\root\cimv2").ExecQuery _'
    '    ("Select * from Win32_Process Where ProcessId = " & Wscript.Arguments(0))'
    'For Each objProcess in colProcessList'
    '    dtmStartTime = objProcess.CreationDate'
    '    Wscript.Echo dtmStartTime'
    'Next'
    };
for i=1:length(wuptime)
    fprintf(fid,'%s\n',wuptime{i});
end
fclose(fid);
command = sprintf('%s %s %s','cscript',filename,pid);
[status, out] = system(command);
delete(filename);
end

function [days,hhmmss] = nixParseTime(out)
% From man ps, etime in the form [[DD-]hh:]mm:ss
hhmmss = regexp(out,'(\d{2}:)?\d{2}:\d{2}','match','once');
days = str2double(regexp(out,'\d+(?=-)','match','once'));
% Same with named tokens capture: regexp(out,'(?<days>\d+(?=-))-?(?<hhmmss>(\d{2}:)?\d{2}:\d{2})','names')
if length(strfind(hhmmss,':')) == 1
    % Uptime less than 1 hour
    hhmmss = ['00:',hhmmss];
end
if isnan(days)
    % Uptime less than 1 day
    days = 0;
end
end

function [days,hhmmss] = winParseTime(out)
% http://technet.microsoft.com/en-us/library/ee156576.aspx
wmidate = regexp(out,'\d+\.\d+\+\d+','match','once');
if isempty(wmidate)
    error('Error parsing the VBS output');
end
year = str2double(wmidate(1:4));
month = str2double(wmidate(5:6));
day = str2double(wmidate(7:8));
hour = str2double(wmidate(9:10));
minute = str2double(wmidate(11:12));
second = str2double(wmidate(13:14));
% Get time in seconds between start time and now
elapsed = floor(etime(clock,[year,month,day,hour,minute,second]));
% Convert seconds to days and hh:mm:ss
ss = mod(elapsed,60);
mm = floor(mod(elapsed/60,60));
hh = floor(mod(elapsed/3600,24));
days = floor(elapsed/86400);
hhmmss = sprintf('%02d:%02d:%02d',hh,mm,ss);
end